@@ -21,13 +21,13 @@ import inspect
 import traceback
 import threading
 
+import testflows.settings as settings
+
 from collections import namedtuple
-from typing import Optional
 from multiprocessing.util import Finalize
 
 from testflows._core.contrib import cloudpickle
 from testflows._core.contrib.aiomsg import Socket
-from testflows._core.exceptions import exception as get_exception
 
 from .asyncio import asyncio, is_running_in_event_loop, CancelledError
 
@@ -95,29 +95,37 @@ class Service:
         self.open = False
         self.lock = asyncio.Lock(loop=self.loop)
 
-    def register(self, obj, sync=None, expose=None):
+    def register(self, obj, sync=None, expose=None, awaited=True):
         """Register object with the service to be by remote services.
         """
+        if isinstance(obj, BaseServiceObject):
+            raise ValueError(f"registering service objects not allowed")
+
         try:
             loop = asyncio.get_running_loop()
         except RuntimeError:
             loop = None
 
-        async def _async_register(sync: bool=False):
-            async with self.lock:
-                if not self.open:
-                    raise ServiceNotRunningError("service is not running")
-                
-                _id = id(obj)
+        def _register(sync: bool=False):
+            if not self.open:
+                raise ServiceNotRunningError("service is not running")
+            
+            _id = id(obj)
 
-                if _id not in self.objects:
-                    self.objects[_id] = Service.ObjectItem(obj)
+            if _id not in self.objects:
+                self.objects[_id] = Service.ObjectItem(obj)
 
             if sync:
                 return ServiceObject(obj, address=self.address, expose=expose)
             return AsyncServiceObject(obj, address=self.address, expose=expose)
 
-        if loop is None:
+        async def _async_register(sync: bool=False):
+            return _register(sync=sync)
+
+        if loop is self.loop and not awaited:
+            return _register(sync=sync)
+
+        if loop is None or not awaited:
             return asyncio.run_coroutine_threadsafe(_async_register(sync=True), loop=self.loop).result()
         elif loop is not self.loop:
             return asyncio.wrap_future(asyncio.run_coroutine_threadsafe(_async_register(sync=sync), loop=self.loop))
@@ -134,11 +142,10 @@ class Service:
             loop = None
 
         async def _async_unregister():
-            async with self.lock:
-                _id = id(obj)
+            _id = id(obj)
 
-                if _id in self.objects:
-                    del self.objects[id(obj)]
+            if _id in self.objects:
+                del self.objects[id(obj)]
         
         if loop is None:
             return asyncio.run_coroutine_threadsafe(_async_unregister(), loop=self.loop).result()
@@ -167,10 +174,34 @@ class Service:
             """
             await asyncio.wait_for(self.connections[address].wait(), timeout=timeout)
 
-            await asyncio.wait_for(self.out_socket.send_pickle(
-                    obj=(self.MsgTypes.REQUEST, rid, (oid, fn, args, kwargs)),
-                    identity=self.connections[address].identity
-                ), timeout=timeout)
+            try:
+                await asyncio.wait_for(self.out_socket.send_pickle(
+                        obj=(self.MsgTypes.REQUEST, rid, (oid, fn, args, kwargs)),
+                        identity=self.connections[address].identity
+                    ), timeout=timeout)
+            except TypeError:
+                # convert any unpicklable objects to service objects
+                pickler = self.out_socket.pickler
+
+                args = list(args)
+                for i in range(len(args)):
+                    arg = args[i]
+                    try:
+                        pickler.dumps(arg)
+                    except TypeError:
+                        args[i] = await self.register(arg, sync=True)
+                
+                for k in kwargs:
+                    arg = kwargs[k]
+                    try:
+                        pickler.dumps(arg)
+                    except TypeError:
+                        kwargs[k] = await self.register(arg, sync=True)
+
+                await asyncio.wait_for(self.out_socket.send_pickle(
+                        obj=(self.MsgTypes.REQUEST, rid, (oid, fn, args, kwargs)),
+                        identity=self.connections[address].identity
+                    ), timeout=timeout)
 
             if resp:
                 event = asyncio.Event()
@@ -348,22 +379,6 @@ def process_service(**kwargs):
     """Get or create global process wide object service.
     """
     global _process_service
-
-    running_in_loop = is_running_in_event_loop()
-
-    async def _async_start_service(**kwargs):
-        """Start process service.
-        """
-        global _process_service
-        _process_service = await Service(**kwargs).__aenter__()
-        atexit.register(_stop_process_service)
-        return _process_service
-
-    async def _async_process_service():
-        """Return process service.
-        """
-        return _process_service
-
     with _process_service_lock:
         if _process_service is None:
             loop = kwargs.pop("loop", None)
@@ -394,15 +409,9 @@ def process_service(**kwargs):
             kwargs["loop"] = loop
             kwargs["name"] = kwargs.get("name", f"process-service-{os.getpid()}")
 
-            if running_in_loop:
-                return _async_start_service(**kwargs)
-
             _process_service = Service(**kwargs).__enter__()
             atexit.register(_stop_process_service)
 
-        if running_in_loop:
-            return _async_process_service()
-
     return _process_service
 
 
@@ -442,6 +451,12 @@ class BaseServiceObject:
             exitpriority=1
             )
 
+    def __str__(self):
+        return f"{self.__class__.__name__}:0x{self.oid:x},{self.address}] object at 0x{id(self):x}"
+
+    def __repr__(self):
+        return str(self)
+
     @staticmethod
     def _incref(oid, address):
         """Increment service object reference count.
@@ -455,9 +470,15 @@ class BaseServiceObject:
         BaseServiceObject.__proxy_call__(oid, address, "__incref__")
 
     @staticmethod
-    def _decref(oid, address):
+    def _decref(oid, address, _timeout_err={}, _service_not_running_err=[False]):
         """Decrement service object reference count.
         """
+        if _service_not_running_err[0]:
+            return
+
+        if _timeout_err.get(address):
+            return
+
         try:
             if is_running_in_event_loop():
                 if _process_service: 
@@ -466,8 +487,15 @@ class BaseServiceObject:
                         return
 
             BaseServiceObject.__proxy_call__(
-                oid, address, "__decref__", timeout=0.1)
-        except (TimeoutError, ServiceObjectNotFoundError, ServiceNotRunningError, CancelledError):
+                oid, address, "__decref__", timeout=settings.service_timeout)
+
+        except TimeoutError:
+            _timeout_err[address] = True
+
+        except ServiceNotRunningError:
+            _service_not_running_err[0] = True
+
+        except (ServiceObjectNotFoundError, CancelledError):
             pass
 
     def __eq__(self, other: object) -> bool:
@@ -484,26 +512,28 @@ class BaseServiceObject:
         if kwargs is None:
             kwargs = dict()
 
-        if _process_service is None:
-            raise ServiceNotRunningError("service has not been started")
+        try:
+            async def wrap(c):
+                """Wrap coroutine that is running in another event loop.
+                """
+                if asyncio.get_event_loop() is not _process_service.loop:
+                    return await asyncio.wrap_future(asyncio.run_coroutine_threadsafe(c, loop=_process_service.loop))
+                return await c
 
-        async def wrap(c):
-            """Wrap coroutine that is running in another event loop.
-            """
-            if asyncio.get_event_loop() is not _process_service.loop:
-                return await asyncio.wrap_future(asyncio.run_coroutine_threadsafe(c, loop=_process_service.loop))
-            return await c
+            send = await wrap(_process_service._connect(address))
+            response = await wrap(send(uuid.uuid1().bytes, oid, fn, args, kwargs, resp=True, timeout=timeout))
 
-        send = await wrap(_process_service._connect(address))
-        response = await wrap(send(uuid.uuid1().bytes, oid, fn, args, kwargs, resp=True, timeout=timeout))
+            await asyncio.wait_for(wrap(response.wait()), timeout=timeout)
+            reply_type, reply_body = response.message
 
-        await asyncio.wait_for(wrap(response.wait()), timeout=timeout)
-        reply_type, reply_body = response.message
-
-        if reply_type == Service.MsgTypes.REPLY_EXCEPTION:
-            if isinstance(reply_body, (SystemExit, KeyboardInterrupt)):
-                reply_body = RuntimeError(f"{reply_body.__class__}: {reply_body}")
-            raise reply_body
+            if reply_type == Service.MsgTypes.REPLY_EXCEPTION:
+                if isinstance(reply_body, (SystemExit, KeyboardInterrupt)):
+                    reply_body = RuntimeError(f"{reply_body.__class__}: {reply_body}")
+                raise reply_body
+        except AttributeError:
+            if _process_service is None:
+                raise ServiceNotRunningError("service has not been started")
+            raise
 
         return reply_body
 
@@ -511,7 +541,12 @@ class BaseServiceObject:
     def __proxy_call__(oid, address, fn, args=None, kwargs=None, timeout=None):
         """Synchronously execute function call on the remote service.
         """
-        return asyncio.run_coroutine_threadsafe(BaseServiceObject.__async_proxy_call__(oid, address, fn, args, kwargs, timeout=timeout), loop=_process_service.loop).result()
+        try:
+            return asyncio.run_coroutine_threadsafe(BaseServiceObject.__async_proxy_call__(oid, address, fn, args, kwargs, timeout=timeout), loop=_process_service.loop).result()
+        except AttributeError:
+            if _process_service is None:
+                raise ServiceNotRunningError("service has not been started")
+            raise
 
     def __reduce__(self):
         """Make service object serializable.
@@ -539,7 +574,7 @@ def RebuildServiceObject(typename, exposed, oid, address, _incref=True):
 def RebuildAsyncServiceObject(typename, exposed, oid, address, _incref=True):
     """Rebuild async service object during unpickling.
     """
-    return ServiceObjectType(typename, exposed, _async=True)(oid=oid, address=address, _incref=_incref)
+    return ServiceObjectType(typename, exposed, asynced=True)(oid=oid, address=address, _incref=_incref)
 
 
 def make_exposed_defs(exposed, asynced):
